using Microsoft.Win32;
using System;
using System.IO;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Text;

namespace System.Drawing
{
	public class FileThumbnail
	{
		private const string GUID_ISHELLFOLDER = "000214E6-0000-0000-C000-000000000046";
		private const string GUID_IEXTRACTIMAGE = "BB2E617C-0920-11d1-9A0B-00C04FC2D6C1";

		public static Bitmap ExtractThumbNail(FileInfo file)
		{
			return ExtractThumbNail(file, new System.Drawing.Size(100, 100));
		}

		public static Bitmap ExtractThumbNail(FileInfo file, System.Drawing.Size size)
		{
			Bitmap thumbnail = null;			
			IMalloc alloc = null;
			IShellFolder folder = null;
			IShellFolder item = null;
			IEnumIDList iidList = null;
			IntPtr pidlFolder = IntPtr.Zero;

			if (file.Exists)
			{
				try
				{
					ShlObj.SHGetMalloc(out alloc);
					ShlObj.SHGetDesktopFolder(out folder);

					if (folder != null)
					{					
						int cParsed = 0;
						int pdwAttrib = 0;
				
						folder.ParseDisplayName(
							IntPtr.Zero, 
							IntPtr.Zero, 
							file.Directory.FullName, 
							out cParsed, 
							out pidlFolder,
							out pdwAttrib);

						if (pidlFolder != IntPtr.Zero)
						{
							Guid iidShellFolder = new Guid(GUID_ISHELLFOLDER);

							folder.BindToObject(pidlFolder, IntPtr.Zero, ref iidShellFolder, ref item);

							if (item != null)
							{
								item.EnumObjects(
									IntPtr.Zero, 
									(SHCONTF.SHCONTF_FOLDERS | SHCONTF.SHCONTF_NONFOLDERS),
									ref iidList);
						
								if (iidList != null)
								{
									IntPtr pidl = IntPtr.Zero;
									int hRes = 0;									
									int fetched = 0;
									bool complete = false;

									while (!complete)
									{
										hRes = iidList.Next(1, ref pidl, out fetched);
										if (hRes != 0)
										{
											pidl = IntPtr.Zero;
											complete = true;
										}
										else
										{
											if (GetThumbNail(ref thumbnail, file, pidl, item, size))
											{
												complete = true;
											}
										}
										if (pidl != IntPtr.Zero)
										{
											alloc.Free(pidl);
										}
									}
								}								
							}
						}
					}
				}
				catch (System.Exception e)
				{
					throw e;
				}
				finally
				{
					if (pidlFolder != IntPtr.Zero) alloc.Free(pidlFolder);
					if (iidList != null)
					{
						Marshal.ReleaseComObject(iidList);
						iidList = null;
					}
					if (item != null)
					{
						Marshal.ReleaseComObject(item);
						item = null;
					}
					if (folder != null)
					{
						Marshal.ReleaseComObject(folder);
						folder = null;
					}
					if (alloc != null)
					{
						Marshal.ReleaseComObject(alloc);
						alloc = null;
					}
				}
			}		
			else
			{
				throw new FileNotFoundException(
					String.Format("The file '{0}' does not exist", file.FullName),
					file.FullName);
			}		
			return thumbnail;
		}

		private static bool GetThumbNail(ref Bitmap thumbnail, FileInfo file, IntPtr pidl, IShellFolder item, Size desiredSize)
		{
			IntPtr hBmp = IntPtr.Zero;
			IExtractImage extractImage = null;
			IUnknown iunk = null;

			try
			{
				string pidlPath = ShlObj.GetPathFromPidl(pidl);
				if (Path.GetFileName(pidlPath).ToUpper().Equals(file.Name.ToUpper()))
				{
					int prgf = 0;
					Guid iidExtractImage = new Guid(GUID_IEXTRACTIMAGE);
					item.GetUIObjectOf(
						IntPtr.Zero, 
						1, 
						ref pidl, 
						ref iidExtractImage,
						out prgf,
						ref iunk);				
					extractImage = (IExtractImage)iunk;
					
					if (extractImage != null)
					{
						StringBuilder location = new StringBuilder(260, 260);

						int priority = 0;
						int requestedColorDepth = 32;
						//int uFlags = (int)(IEIFLAG.IEIFLAG_ASPECT | IEIFLAG.IEIFLAG_SCREEN);
						int uFlags = (int)IEIFLAG.IEIFLAG_ASPECT;
						
						extractImage.GetLocation(
							location, 
							location.Capacity, 
							ref priority,
							ref desiredSize,
							requestedColorDepth,
							ref uFlags);

						extractImage.Extract(out hBmp);						
						if (hBmp != IntPtr.Zero)
						{
							thumbnail = Bitmap.FromHbitmap(hBmp);
						}
					}
					return true;
				}
				else
				{
					return false;
				}
			}
			catch (System.Exception e)
			{
				throw e;
			}
			finally
			{
				if (hBmp != IntPtr.Zero) Wingdi.DeleteObject(hBmp);
				if (iunk != null)
				{
					Marshal.ReleaseComObject(iunk);
					iunk = null;
				}
				if (extractImage != null)
				{
					Marshal.ReleaseComObject(extractImage);
					extractImage = null;
				}
			}
		}
	}
}
